@@ -1,4 +1,4 @@
-/*  $Id: k4start.c 2348 2006-01-02 01:43:25Z rra $
+/*  $Id: k4start.c 2381 2006-01-22 09:01:05Z rra $
 **
 **  Kerberos v4 kinit replacement suitable for daemon authentication.
 **
@@ -80,7 +80,7 @@ extern int mkstemp(char *);
 /* Make sure everything compiles even if no aklog program was found by
    configure. */
 #ifndef PATH_AKLOG
-# define PATH_AKLOG NULL
+# define PATH_AKLOG ""
 #endif
 
 /* Holds the various command-line options for passing to functions. */
@@ -117,6 +117,7 @@ Usage: k4start [options] [name]\n\
    -H <limit>           Check for a happy ticket, one that doesn't expire in\n\
                         less than <limit> minutes, and exit 0 if it's okay,\n\
                         otherwise obtain a ticket\n\
+   -h                   Display this usage message and exit\n\
    -K <interval>        Run as daemon, renew ticket every <interval> minutes\n\
                         (implies -q unless -v is given)\n\
    -k <file>            Use <file> as the ticket cache\n\
@@ -131,7 +132,8 @@ Usage: k4start [options] [name]\n\
 If the environment variable KINIT_PROG is set to a program (such as aklog)\n\
 then this program will automatically be executed after the ticket granting\n\
 ticket has been retrieved unless -n is given.  Otherwise, the default is to\n\
-not run any aklog program.\n";
+not run any aklog program.  If -t is given and KINIT_PROG is not set,\n\
+%s.\n";
 
 
 /*
@@ -159,7 +161,10 @@ die(const char *format, ...)
 static void
 usage(int status)
 {
-    fprintf((status == 0) ? stdout : stderr, usage_message);
+    fprintf((status == 0) ? stdout : stderr, usage_message,
+            ((PATH_AKLOG[0] == '\0')
+             ? "k4start exits with an error"
+             : "the program run will be " PATH_AKLOG));
     exit(status);
 }
 
@@ -262,9 +267,10 @@ main(int argc, char *argv[])
 
     /* Parse command-line options. */
     memset(&options, 0, sizeof(options));
-    while ((opt = getopt(argc, argv, "bf:H:I:i:K:k:l:np:qr:S:stu:v")) != EOF)
+    while ((opt = getopt(argc, argv, "bf:H:hI:i:K:k:l:np:qr:S:stu:v")) != EOF)
         switch (opt) {
         case 'b': background = 1;               break;
+        case 'h': usage(0);                     break;
         case 'k': options.cache = optarg;       break;
         case 'n': options.no_aklog = 1;         break;
         case 'p': pidfile = optarg;             break;
@@ -366,7 +372,7 @@ main(int argc, char *argv[])
         aklog = PATH_AKLOG;
     else
         options.run_aklog = 1;
-    if (aklog == NULL && options.run_aklog && !options.no_aklog)
+    if (aklog[0] == '\0' && options.run_aklog && !options.no_aklog)
         die("set KINIT_PROG to specify the path to aklog");
 
     /* The default username is the name of the local user. */
