@@ -1,4 +1,4 @@
-/*  $Id: k5start.c 2348 2006-01-02 01:43:25Z rra $
+/*  $Id: k5start.c 2553 2006-05-06 05:12:32Z rra $
 **
 **  Kerberos v5 kinit replacement suitable for daemon authentication.
 **
@@ -94,7 +94,7 @@ extern int mkstemp(char *);
 /* Make sure everything compiles even if no aklog program was found by
    configure. */
 #ifndef PATH_AKLOG
-# define PATH_AKLOG NULL
+# define PATH_AKLOG ""
 #endif
 
 /* Holds the various command-line options for passing to functions, after
@@ -130,6 +130,7 @@ Usage: k5start [options] [name [command]]\n\
    -H <limit>           Check for a happy ticket, one that doesn't expire in\n\
                         less than <limit> minutes, and exit 0 if it's okay,\n\
                         otherwise obtain a ticket\n\
+   -h                   Display this usage message and exit\n\
    -K <interval>        Run as daemon, renew ticket every <interval> minutes\n\
                         (implies -q unless -v is given)\n\
    -k <file>            Use <file> as the ticket cache\n\
@@ -144,8 +145,8 @@ Usage: k5start [options] [name [command]]\n\
    -v                   Verbose\n\
 \n\
 If the environment variable KINIT_PROG is set to a program (such as aklog)\n\
-then this program will be executed when requested by the -t flag, rather\n\
-than whatever aklog program was compiled into k5start.\n";
+then this program will be executed when requested by the -t flag.\n\
+Otherwise, %s.\n";
 
 
 /*
@@ -173,7 +174,10 @@ die(const char *format, ...)
 static void
 usage(int status)
 {
-    fprintf((status == 0) ? stdout : stderr, usage_message);
+    fprintf((status == 0) ? stdout : stderr, usage_message,
+            ((PATH_AKLOG[0] == '\0')
+             ? "using -t is an error"
+             : "the default program to run is " PATH_AKLOG));
     exit(status);
 }
 
@@ -388,12 +392,12 @@ main(int argc, char *argv[])
 
     /* Parse command-line options. */
     memset(&options, 0, sizeof(options));
-    while ((opt = getopt(argc, argv, "bf:H:I:i:K:k:l:np:qr:S:stUu:v")) != EOF)
+    while ((opt = getopt(argc, argv, "bf:H:hI:i:K:k:l:np:qr:S:stUu:v")) != EOF)
         switch (opt) {
         case 'b': background = 1;               break;
+        case 'h': usage(0);                     break;
         case 'I': sinst = optarg;               break;
         case 'i': inst = optarg;                break;
-        case 'k': cache = optarg;               break;
         case 'n': /* Ignored */                 break;
         case 'p': pidfile = optarg;             break;
         case 'q': options.quiet = 1;            break;
@@ -419,6 +423,12 @@ main(int argc, char *argv[])
             if (options.keep_ticket <= 0)
                 die("-K interval argument %s out of range", optarg);
             break;
+        case 'k':
+            cache = malloc(strlen(optarg) + strlen("FILE:") + 1);
+            if (cache == NULL)
+                die("cannot allocate memory: %s", strerror(errno));
+            sprintf(cache, "FILE:%s", optarg);
+            break;
         case 'l':
             k5_errno = krb5_string_to_deltat(optarg, &life_secs);
             if (k5_errno != 0 || life_secs == 0)
@@ -472,7 +482,7 @@ main(int argc, char *argv[])
     options.aklog = getenv("KINIT_PROG");
     if (options.aklog == NULL)
         options.aklog = PATH_AKLOG;
-    if (options.aklog == NULL && options.run_aklog)
+    if (options.aklog[0] == '\0' && options.run_aklog)
         die("set KINIT_PROG to specify the path to aklog");
 
     /* Establish a K5 context. */
@@ -502,6 +512,8 @@ main(int argc, char *argv[])
         int fd;
 
         cache = malloc(strlen("/tmp/krb5cc__XXXXXX") + 20 + 1);
+        if (cache == NULL)
+            die("cannot allocate memory: %s", strerror(errno));
         sprintf(cache, "/tmp/krb5cc_%d_XXXXXX", (int) getuid());
         fd = mkstemp(cache);
         if (fd < 0)
@@ -633,6 +645,13 @@ main(int argc, char *argv[])
         }
     }
 
+    /* If we're backgrounded, check our ticket and possibly do the
+       authentication again.  Normally this will never trigger, but on Mac OS
+       X our ticket cache isn't going to survive the setsid that daemon does
+       and we've now lost our credentials. */
+    if (background && ticket_expired(ctx, &options))
+        status = authenticate(ctx, &options);
+
     /* Spawn the external command, if we were told to run one. */
     if (command != NULL) {
         child = start_command(command[0], command);
